from .tokens import TokenType, Token
from .lexer import Lexer


class InterpreterException(Exception):
    pass


class Interpreter():
    def __init__(self):
        self._current_token: Token = None
        self._lexer = Lexer()

    def _check_token_type(self, type_: TokenType):
        if self._current_token.type_ == type_:
            self._current_token = self._lexer.next()
        else:
            raise InterpreterException("invalid token order")

    def _expr(self) -> float:
        self._current_token = self._lexer.next()  # берём первый токен
        left = self._current_token
        self._check_token_type(TokenType.FLOAT)
        op = self._current_token
        if op.type_ == TokenType.PLUS:
            self._check_token_type(TokenType.PLUS)
        else:
            self._check_token_type(TokenType.MINUS)
        right = self._current_token
        self._check_token_type(TokenType.FLOAT)
        if op.type_ == TokenType.PLUS:
            return float(left.value) + float(right.value)
        elif op.type_ == TokenType.MINUS:
            return float(left.value) - float(right.value)
        raise InterpreterException("bad operator")

    def __call__(self, text: str):
        return self.interpret(text)

    def interpret(self, text: str) -> float:
        self._lexer.init(text)
        return self._expr()
